import {
  noop,
  className,
  mergeData,
  renderRules,
  validate
} from '../../core/utils'
import { BUTTON, COLOR, ICON } from '../../core/constants'
import Icon from '../icon/icon'
import Text from '../text/text'
import rules from './button.rules'

const SIZES = [ 'small', 'large' ]
const DISPLAYS = [ 'solid', 'stroke' ]

export default {
  functional: true,
  inject: [
    'fela',
    'croudClassName'
  ],
  props: {
    tag: {
      type: String,
      default: 'button'
    },
    text: {
      type: String
    },
    disabled: {
      type: Boolean
    },
    outline: {
      type: Boolean
    },
    uppercase: {
      type: Boolean
    },
    icon: {
      type: String
    },
    click: {
      type: Function
    },
    color: {
      type: String,
      default: COLOR.TEAL
    },
    size: {
      type: String,
      enums: SIZES,
      default: 'small',
      validator: validate(SIZES)
    },
    display: {
      type: String,
      enums: DISPLAYS,
      default: 'stroke',
      validator: validate(DISPLAYS)
    }
  },
  render(h, ctx) {
    const {
      text,
      icon,
      size,
      color,
      display,
      disabled,
      uppercase,
      click = noop
    } = ctx.props

    const isSmall = size === 'small'
    const isSolid = display === 'solid'
    const textSize = isSmall ? 13 : 14
    const iconSize = isSmall ? ICON.MICRO : ICON.SMALL
    const iconColor = isSolid ? COLOR.WHITE : color
    const textColor = isSolid ? COLOR.WHITE : color
    const outerSize = isSmall ? BUTTON.HEIGHT_SMALL : BUTTON.HEIGHT_LARGE
    const innerSize = outerSize - (BUTTON.BORDER_WIDTH * 2)

    const classes = renderRules(ctx, rules, Object.assign({
      isSmall,
      isSolid,
      textSize,
      iconSize,
      outerSize,
      innerSize
    }, ctx.props))

    return h(ctx.props.tag, mergeData(ctx, {
      class: className(ctx, 'button', classes.button),
      props: { disabled },
      on: { click }
    }), [
      icon ? h(Icon, {
        class: className(ctx, 'button-icon', classes.icon),
        props: {
          color: iconColor,
          size: iconSize,
          name: icon
        }
      }) : null,
      text ? h(Text, {
        class: className(ctx, 'button-text', classes.text),
        props: {
          color: textColor,
          size: textSize,
          text: text,
          uppercase
        }
      }) : null
    ])
  }
}
