import * as _ from './utils'

test('complement(fn)', () => {
  const isLetterA = (letter) => letter === 'a'
  expect(isLetterA()).toBe(false)
  expect(isLetterA('b')).toBe(false)
  expect(isLetterA('a')).toBe(true)

  const isNotLetterA = _.complement(isLetterA)
  expect(isNotLetterA).toEqual(expect.any(Function))
  expect(isNotLetterA()).toBe(true)
  expect(isNotLetterA('b')).toBe(true)
  expect(isNotLetterA('a')).toBe(false)
})

test('validate(values)', () => {
  const validator = _.validate([ 1, 2 ])
  expect(validator).toEqual(expect.any(Function))
  expect(validator(1)).toBe(true)
  expect(validator(2)).toBe(true)
  expect(validator(3)).toBe(false)
})

test('ifElse(condition)', () => {
  const ifTrue = _.ifElse(true)
  const ifFalse = _.ifElse(false)
  expect(ifTrue).toEqual(expect.any(Function))
  expect(ifFalse).toEqual(expect.any(Function))
  expect(ifTrue('a', 'b')).toBe('a')
  expect(ifFalse('a', 'b')).toBe('b')
  expect(ifFalse('a')).toBeUndefined()
})

test('isMeta(string)', () => {
  expect(_.isMeta('meta')).toBe(true)
  expect(_.isMeta('h6')).toBe(false)
  expect(_.isMeta('h')).toBe(false)
  expect(_.isMeta('h7')).toBe(false)
  expect(_.isMeta('span')).toBe(false)
})

test('isHeading(string)', () => {
  expect(_.isHeading('h1')).toBe(true)
  expect(_.isHeading('h6')).toBe(true)
  expect(_.isHeading('h')).toBe(false)
  expect(_.isHeading('h7')).toBe(false)
  expect(_.isHeading('span')).toBe(false)
})

test('isType(type)', () => {
  const isBoolean = _.isType('boolean')
  expect(isBoolean).toEqual(expect.any(Function))
  expect(isBoolean(false)).toBe(true)
  expect(isBoolean(true)).toBe(true)
  expect(isBoolean('a')).toBe(false)
  expect(isBoolean()).toBe(false)
})

test('isNull(value)', () => {
  expect(_.isNull()).toBe(false)
  expect(_.isNull(0)).toBe(false)
  expect(_.isNull('')).toBe(false)
  expect(_.isNull(true)).toBe(false)
  expect(_.isNull(false)).toBe(false)
  expect(_.isNull(null)).toBe(true)
})

test('isRegex(value)', () => {
  expect(_.isRegex()).toBe(false)
  expect(_.isRegex(0)).toBe(false)
  expect(_.isRegex('')).toBe(false)
  expect(_.isRegex(true)).toBe(false)
  expect(_.isRegex(null)).toBe(false)
  expect(_.isRegex(/a/)).toBe(true)
  expect(_.isRegex(new RegExp('a'))).toBe(true)
})

test('isNumber(value)', () => {
  expect(_.isNumber()).toBe(false)
  expect(_.isNumber('')).toBe(false)
  expect(_.isNumber(null)).toBe(false)
  expect(_.isNumber(true)).toBe(false)
  expect(_.isNumber(false)).toBe(false)
  expect(_.isNumber(0)).toBe(true)
  expect(_.isNumber(1)).toBe(true)
})

test('isString(value)', () => {
  expect(_.isString()).toBe(false)
  expect(_.isString(0)).toBe(false)
  expect(_.isString(null)).toBe(false)
  expect(_.isString(true)).toBe(false)
  expect(_.isString('')).toBe(true)
})

test('isFunction(value)', () => {
  const arrowFn = (a) => a
  function normalFn(a) { return a }
  expect(_.isFunction()).toBe(false)
  expect(_.isFunction('')).toBe(false)
  expect(_.isFunction(null)).toBe(false)
  expect(_.isFunction(true)).toBe(false)
  expect(_.isFunction(false)).toBe(false)
  expect(_.isFunction(0)).toBe(false)
  expect(_.isFunction(1)).toBe(false)
  expect(_.isFunction(normalFn)).toBe(true)
  expect(_.isFunction(arrowFn)).toBe(true)
})

test('isUndefined(value)', () => {
  expect(_.isUndefined(0)).toBe(false)
  expect(_.isUndefined('0')).toBe(false)
  expect(_.isUndefined(null)).toBe(false)
  expect(_.isUndefined(false)).toBe(false)
  expect(_.isUndefined(undefined)).toBe(true) // eslint-disable-line
  expect(_.isUndefined()).toBe(true)
})

test('isDefined(value)', () => {
  expect(_.isDefined(0)).toBe(true)
  expect(_.isDefined('0')).toBe(true)
  expect(_.isDefined(null)).toBe(true)
  expect(_.isDefined(false)).toBe(true)
  expect(_.isDefined(undefined)).toBe(false) // eslint-disable-line
  expect(_.isDefined()).toBe(false)
})

test('isLinkActive(linkPath, routePath, exact)', () => {
  expect(_.isLinkActive('/foo', '/bar', false)).toBe(false)
  expect(_.isLinkActive('/foo', '/bar', true)).toBe(false)
  expect(_.isLinkActive('/foo', '/foo', false)).toBe(true)
  expect(_.isLinkActive('/foo', '/foo', true)).toBe(true)
  expect(_.isLinkActive('/', '/foo', false)).toBe(true)
  expect(_.isLinkActive('/', '/foo', true)).toBe(false)
})

test('offset(outer, inner)', () => {
  expect(_.offset(10, 8)).toBe(1)
  expect(_.offset(10, 6)).toBe(2)
  expect(_.offset(8, 10)).toBe(-1)
})

test('units(count)', () => {
  expect(_.units()).toBe(8)
  expect(_.units(1)).toBe(8)
  expect(_.units(2)).toBe(16)
})

test('hexToRgbMap(hex)', () => {
  expect(_.hexToRgbMap()).toBeNull()
  expect(_.hexToRgbMap('0')).toBeNull()

  const black = { r: 0, g: 0, b: 0 }
  expect(_.hexToRgbMap('#000000')).toEqual(black)
  expect(_.hexToRgbMap('000000')).toEqual(black)
  expect(_.hexToRgbMap('#000')).toEqual(black)
  expect(_.hexToRgbMap('000')).toEqual(black)

  const red = { r: 255, g: 0, b: 0 }
  expect(_.hexToRgbMap('#FF0000')).toEqual(red)
  expect(_.hexToRgbMap('FF0000')).toEqual(red)
  expect(_.hexToRgbMap('#F00')).toEqual(red)
  expect(_.hexToRgbMap('F00')).toEqual(red)
})

test('hexToRgb(hex)', () => {
  expect(_.hexToRgb('#000000')).toBe('rgb(0,0,0)')
  expect(_.hexToRgb('#FF0000')).toBe('rgb(255,0,0)')
})

test('hexToRgba(hex, alpha)', () => {
  expect(_.hexToRgba('#000000', 0.5)).toBe('rgba(0,0,0,0.5)')
  expect(_.hexToRgba('#FF0000', 0.5)).toBe('rgba(255,0,0,0.5)')
})

test('camelToKebab(string)', () => {
  expect(_.camelToKebab('fooBar')).toBe('foo-bar')
  expect(_.camelToKebab('FooBar')).toBe('foo-bar')
})

test('kebabToCamel(string)', () => {
  expect(_.kebabToCamel('foo')).toBe('foo')
  expect(_.kebabToCamel('foo-bar-baz')).toBe('fooBarBaz')
})

test('capitalize(string)', () => {
  expect(_.capitalize('FOO')).toBe('Foo')
  expect(_.capitalize('foo')).toBe('Foo')
  expect(_.capitalize('fOO')).toBe('Foo')
})

test('wrapKey(key, prefix, postfix)', () => {
  expect(_.wrapKey('foo')).toBe('foo')
  expect(_.wrapKey('foo', 'before')).toBe('beforeFoo')
  expect(_.wrapKey('foo', null, 'after')).toBe('fooAfter')
  expect(_.wrapKey('foo', 'before', 'after')).toBe('beforeFooAfter')
})

test('parseEdges(edges)', () => {
  const edges = [ 'left', 'right' ]
  expect(_.parseEdges()).toBeUndefined()
  expect(_.parseEdges(edges)).toBe(edges)
  expect(_.parseEdges('left|right')).toEqual(edges)
})

test('attempt(fn, ...args)', () => {
  const fn = jest.fn()
  _.attempt(fn, 1, 2, 3)
  expect(fn).toHaveBeenCalledWith(1, 2, 3)
})
